//------------------------------------------------------------------
#property copyright "mladen"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_plots   4
#property indicator_label1  "Smooth bulls"
#property indicator_type1   DRAW_LINE
#property indicator_color1  LightBlue
#property indicator_width1  2 
#property indicator_label2  "Smooth bears"
#property indicator_type2   DRAW_LINE
#property indicator_color2  Tomato
#property indicator_width2  2
#property indicator_label3  "Signal bulls"
#property indicator_type3   DRAW_LINE
#property indicator_color3  Aqua
#property indicator_style3  STYLE_DOT
#property indicator_label4  "Signal bears"
#property indicator_type4   DRAW_LINE
#property indicator_color4  Orange
#property indicator_style4  STYLE_DOT

//
//
//
//
//

enum ENUM_AS_MODE
{
   RSI_method,        // Calculate absolute strength using RSI method
   Stochastic_method  // Calculate absolute strength using Stochastic method
};
enum enAverages
{
   ma_sma,        // Simple moving average
   ma_ema,        // Eponential moving average
   ma_smma,       // Smoothed moving average
   ma_lwma        // Linear weighted moving average
};
enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average     // Average (high+low+oprn+close)/4
};

//
//
//
//
//

input ENUM_AS_MODE Mode      = RSI_method; // Calculation method
input int          Length    = 10;         // Calculation length
input int          Smooth    = 5;          // Smoothing length
input int          Signal    = 5;          // Signal smoothing length
input enPrices     Price     = pr_close;   // Price
input enAverages   AvgMethod = ma_ema;     // Averaging method

double SmthBulls[];
double SmthBears[];
double SigBulls[];
double SigBears[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

int OnInit()
{
   SetIndexBuffer(0,SmthBulls,INDICATOR_DATA);
   SetIndexBuffer(1,SmthBears,INDICATOR_DATA);
   SetIndexBuffer(2,SigBulls ,INDICATOR_DATA);
   SetIndexBuffer(3,SigBears ,INDICATOR_DATA);
   IndicatorSetString(INDICATOR_SHORTNAME,"Absolute strength ("+(string)Mode+","+(string)Length+","+(string)Smooth+","+(string)Signal+","+(string)Price+")");
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
{

   for (int i=(int)MathMax(prev_calculated-1,1); i<rates_total; i++)
   {
      double Price1 = getPrice(Price,open,close,high,low,rates_total,i);
      double Price2 = getPrice(Price,open,close,high,low,rates_total,i-1);
      double Bulls  = 0;
      double Bears  = 0; 
      
         if (Mode==RSI_method)
         {
            Bulls = 0.5*(MathAbs(Price1-Price2)+(Price1-Price2));
            Bears = 0.5*(MathAbs(Price1-Price2)-(Price1-Price2));
         }
         else
         {
            double smax = high[i]; for(int k=1; k<Length && (i-k)>=0; k++) smax = MathMax(smax,high[i-k]);
            double smin =  low[i]; for(int k=1; k<Length && (i-k)>=0; k++) smin = MathMin(smin, low[i-k]);
            Bulls = Price1 - smin;
            Bears = smax - Price1;
         }
         double AvgBulls = iCustomMa(Bulls,Length,AvgMethod,rates_total,i,0);
         double AvgBears = iCustomMa(Bears,Length,AvgMethod,rates_total,i,1);
                SmthBulls[i] = iCustomMa(AvgBulls,Smooth,AvgMethod,rates_total,i,2);
                SmthBears[i] = iCustomMa(AvgBears,Smooth,AvgMethod,rates_total,i,3);
                SigBulls[i]  = iCustomMa(SmthBulls[i],Signal,AvgMethod,rates_total,i,4);
                SigBears[i]  = iCustomMa(SmthBears[i],Signal,AvgMethod,rates_total,i,5);
   }
   return(rates_total);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double getPrice(enPrices price, const double& open[], const double& close[], const double& high[], const double& low[], int bars, int i)
{
   switch (price)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

#define _maWorkBufferx1 6
#define _maWorkBufferx2 12
#define _maWorkBufferx3 18

double iCustomMa( double price, double length, int mode, int bars, int i, int instanceNo=0)
{
   switch (mode)
   {
      case ma_sma  : return(iSma(price ,(int)length,bars,i,instanceNo));
      case ma_ema  : return(iEma(price ,     length,bars,i,instanceNo));
      case ma_smma : return(iSmma(price,     length,bars,i,instanceNo));
      case ma_lwma : return(iLwma(price,     length,bars,i,instanceNo));
      default      : return(0);
   }
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double workSma[][_maWorkBufferx2];
double iSma(double price, int period,int bars, int r, int instanceNo=0)
{
   if (ArrayRange(workSma,0)!= bars) ArrayResize(workSma,bars); instanceNo *= 2;

   //
   //
   //
   //
   //
      
   int k;      
   workSma[r][instanceNo] = price;
   if (r>=period)
          workSma[r][instanceNo+1] = workSma[r-1][instanceNo+1]+(workSma[r][instanceNo]-workSma[r-period][instanceNo])/period;
   else { workSma[r][instanceNo+1] = 0; for(k=0; k<period && (r-k)>=0; k++) workSma[r][instanceNo+1] += workSma[r-k][instanceNo];  
          workSma[r][instanceNo+1] /= k; }
   return(workSma[r][instanceNo+1]);
}

//
//
//
//
//

double workEma[][_maWorkBufferx1];
double iEma(double price, double period,int bars, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= bars) ArrayResize(workEma,bars);

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
   if (r>0)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   else   workEma[r][instanceNo] = price;
   return(workEma[r][instanceNo]);
}

//
//
//
//
//

double workSmma[][_maWorkBufferx1];
double iSmma(double price, double period,int bars, int r, int instanceNo=0)
{
   if (ArrayRange(workSmma,0)!= bars) ArrayResize(workSmma,bars);

   //
   //
   //
   //
   //

   if (r<period)
         workSmma[r][instanceNo] = price;
   else  workSmma[r][instanceNo] = workSmma[r-1][instanceNo]+(price-workSmma[r-1][instanceNo])/period;
   return(workSmma[r][instanceNo]);
}

//
//
//
//
//

double workLwma[][_maWorkBufferx1];
double iLwma(double price, double period,int bars, int r, int instanceNo=0)
{
   if (ArrayRange(workLwma,0)!= bars) ArrayResize(workLwma,bars);
   
   //
   //
   //
   //
   //
   
   workLwma[r][instanceNo] = price;
      double sumw = period;
      double sum  = period*price;

      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = period-k;
                sumw  += weight;
                sum   += weight*workLwma[r-k][instanceNo];  
      }             
      return(sum/sumw);
}